
# q-expansion of newform 150.12.c.b, downloaded from the LMFDB on 27 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 150
weight = 12
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[101, [1, 0]], [127, [-1, 0]]]
aps_data = [[0, 32], [0, 243], [0, 0], [0, 50008], [-531420, 0], [0, 1332566], [0, 5109678], [-2901404, 0], [0, 30597000], [77006634, 0], [-239418352, 0], [0, 785041666], [411252954, 0], [0, 351233348], [0, -95821680], [0, -1465857378], [-5621152020, 0], [-10473587770, 0], [0, -4515307532], [-8509579560, 0], [0, 2012496986], [22238409568, 0], [0, 6328647516], [50123706678, 0], [0, -94805961314], [13795202478, 0], [0, 65183649128], [0, -93339866916], [151369448650, 0], [0, 237348947346], [0, -514414073360], [298571769420, 0], [0, 616116174918], [522813911356, 0], [1270146260514, 0], [1253711469848, 0], [0, -606707569574], [0, -1584008507572], [0, -2952724162200], [0, -2654101539546], [4064495156964, 0], [-1680732212146, 0], [409135210176, 0], [0, 4719056206274], [0, 3608988480882], [-1721399536712, 0], [1703445809660, 0], [0, -14625286013200], [0, -3364566476556], [-1899203532830, 0], [0, 1519014596346], [-8314965823056, 0], [-13012622473198, 0], [-18713802990924, 0], [0, 5484268053246], [0, 19594148973240], [-35702898898086, 0], [36038674743104, 0], [0, 40730388976786], [4088261050698, 0], [0, -13355500432204], [0, 21943343955822], [0, -252176931740], [-49575205201080, 0], [0, 32431810241066], [0, 95642176049034], [-121579047495868, 0], [0, 83809620132238], [0, 77945675395116], [-112178870095910, 0], [0, -159720754812702], [2463779994696, 0], [0, -134080304339264], [0, -125684353520722], [-56230100344244, 0], [0, 107825843218560], [-237628966425006, 0], [0, -285969574353206], [-93170655151086, 0], [-256949937683594, 0], [296091363538740, 0], [-340485060693730, 0], [137788674940080, 0], [0, 352376754089618], [429184982726248, 0], [0, -361081178389644], [6276881514654, 0], [0, -460906541857562], [797398572217350, 0], [0, -580897190893312], [0, 822532069976292], [488234955785856, 0], [0, 29728121429176], [816135007924644, 0], [-609916588495964, 0], [0, 240472414055592], [156719890564554, 0], [-100578608126982, 0], [0, 1224425212010660], [-362345490584890, 0], [0, -1326304085946764], [0, 2244739616558874], [0, -1101778600026756], [-1691074659274122, 0], [-2365746586247980, 0], [0, 325901835908926], [0, 2766839563505724], [0, 1959660392281650], [3373926699822456, 0], [3135854548398218, 0], [0, -3675681128418992], [0, -978885934529794], [0, 1462553585594310], [-4301011527851876, 0], [-595346430921928, 0], [-751261899449310, 0], [0, 1461443871913388], [0, 3578616021301992], [0, 731395394414214], [1699870896228228, 0], [-178604343718066, 0], [0, -926675672044702], [0, 149664405584562], [0, 6748556779434564], [-1612717768694596, 0], [416261248516854, 0], [2829722232869698, 0], [-6811222080684240, 0], [0, -2731911221435864], [0, -6583680329025466], [-4375643459996972, 0], [0, 4900592000047128], [-704936008362880, 0], [0, -5255155388235854], [3813359614118538, 0], [9812412425812222, 0], [0, 7091357397610446], [0, 2463997954517668], [0, -3049360148357430], [3922651912372326, 0], [17501217622441796, 0], [4481484718794078, 0], [0, -2388191905157320], [0, -3422454871430484], [-7188806436234662, 0], [-12510074262400536, 0], [0, 5885164723051022], [0, -2579952118898154], [3148742293288876, 0], [0, -22853895216247008], [0, 18369539762727466], [-11750671469714094, 0], [0, -7767911723841412], [0, -4627495534783560], [0, -19251116224247588], [-18009829098648528, 0], [5428822550168104, 0], [-27631021175735106, 0], [0, 24605982978712198], [31189203822657318, 0], [0, 31718935020484068], [0, 9409924474322058], [0, 12773086132968568], [35110502073813924, 0], [0, 50866832186427054], [0, -42567618865546392], [22412352862092944, 0], [0, 20326767928668418]]
